import json
import ccxt
import time
import config
import datetime
from flask import request,jsonify
from requests import get

def get_exchange():
    print('• Initializing Exchange Connection')
    # exchange = ccxt.binanceus({
        'apiKey': config.binance_api_key,
        'secret': config.binance_api_secret,
        'timeout': 30000,
        'enableRateLimit': True,
        'options': { 'defaultType': 'spot' },
        'limits': {
            'amount': {
                'min': 0.0000001,
                'max': 100000000,
                }
        }
    })
    exchange.set_sandbox_mode(False)
    print('• Connection Successfull')
    print('------------------------------')
    return exchange

def ip_address():
    if 'X-Forwarded-For' in request.headers:
        proxy_data = request.headers['X-Forwarded-For']
        ip_list = proxy_data.split(',')
        user_ip = ip_list[0]  # first address in list is User IP
        user_ip = user_ip.replace(".","")
    else:
        user_ip = request.remote_addr  # For local development
        user_ip = user_ip.replace(".","")
    return user_ip


def get_symbol_name(symbol):
    try:
        if symbol.endswith('PERP'):
            return symbol.rstrip('PERP')
        if symbol.endswith('.P'):
            return symbol.rstrip('.P')
        return symbol
    except:
        return ''

def fetch_buy_balance(exchange,balance_params,currency,rate):
    balances = exchange.fetchBalance(balance_params)
    balance = balances[currency]['free']
    buy_bal = balance/rate
    return buy_bal

def fetch_sell_balance(exchange,coin_symbol,balance_params):
    base_pair = coin_symbol
    print('• Calculating Quantity')
    start_time = time.time()
    sell_balances = exchange.fetchBalance(balance_params)
    sell_bal = sell_balances[base_pair]['free']
    total_time = round((time.time()-start_time),3)
    print(f'• API Took {total_time} Seconds to Fetch Balance From Exchange')
    quantity = sell_bal
    return quantity

def fetch_rates(exchange,symbol):
    tick = exchange.fetchTicker(symbol)
    mid_point = tick['bid']
    return mid_point

def fetch_balance_params(market):
    if market == "spot":
        balance_params = {'type':'spot'}
    elif market == "future":
        balance_params = {'type':'future'}
    return balance_params


def perc_free_bal(exchange,balance_params,currency,quantity,rate):
    print('• Calculating Quantity')
    start_time = time.time()
    balances = exchange.fetchBalance(balance_params)
    balance = balances[currency]['free']
    total_time = round((time.time()-start_time),3)
    print(f'• API Took {total_time} Seconds to Fetch Balance From Exchange')
    balance = int(balance)
    percentage = balance / 100
    percentage = percentage * int(quantity)
    perc_free_bal = percentage/(float(rate)) 
    perc_free_bal = round(perc_free_bal,8)
    print(f'• Quantity Calculated : {perc_free_bal}')
    print('--------------------------------')
    return perc_free_bal,balance

def fetch_position(exchange,symbol):
    pos = exchange.fetch_balance({'type':'future'})
    pos = pos['info']['positions']
    for data in pos:
        if data['symbol'] == symbol:
            last_pos = data['positionAmt']
    return last_pos

def fetch_entry_price(exchange,symbol):
    pos = exchange.fetch_balance({'type': 'future'})
    if pos is not None:
        pos = pos['info']['positions']
        for data in pos:
            if data['symbol'] == symbol:
                entry_price = data['entryPrice']
    return float(entry_price)

def fetch_balance(exchange,balance_params,currency):
    balances = exchange.fetchBalance(balance_params)
    balance = balances[currency]['free']
    return balance

def fetch_pos_price(exchange,trade_symbol):
    trade_price = json.dumps(exchange.fetchTrades(trade_symbol ))
    trade_price = json.loads(trade_price)
    for data in reversed(trade_price):
        x = data['price']
        break
    trade_price = x
    return trade_price

def read(file_name):
    file = open(file_name, "r+")
    read_file = file.read()
    file.close()
    return read_file


def write(file_name, data):
    file = open(file_name, "w")
    file.write(str(data))
    file.close()

def last_trade_info(user,symbol):
    # user = get_exchange(config.api_key, config.api_secret)
    # #trades = user.fetch_positions(['BTCUSDT','ETHUSDT'])
    trades = user.fetch_my_trades(symbol)

    for data in reversed(trades):
        #print(data)
        info = data['info']
        timestamp_sec = int(info['time']) / 1000
        dt = datetime.datetime.fromtimestamp(timestamp_sec)
        readable_time = str(dt.strftime('%Y-%m-%d %H:%M:%S'))
        side = info['side']
        symbol = info['symbol']
        qty = info['qty']
        price = info['price']
        rPnl = round(float(info['realizedPnl']),5)
        margin_asset = info['marginAsset']
        commission = round(float(info['commission']),5)
        commission_asset = info['commissionAsset']
        nPnL = float(rPnl) - float(commission)
        
        if nPnL < 0:
            status = 'LOSS'
            l_emoji = '⛔'
        else:
            status = 'PROFIT'
            l_emoji = '✳'
        if nPnL == 0.0:
            status = 'FLAT'
            l_emoji = '💠'
        
        break

    return readable_time,side,symbol,price,margin_asset,commission,commission_asset,rPnl,nPnL,status,l_emoji,qty

def update_config(index_1,value_1,index_2,value_2):
    with open('config.py', 'r') as file:
      lines = file.readlines()
  
    modified_lines = []
    for line in lines:
        if line.startswith(index_1):
            modified_lines.append(f"{index_1}'{value_1}'\n")
        elif line.startswith(index_2):
            modified_lines.append(f"{index_2}'{value_2}'\n")
        else:
            modified_lines.append(line)
    
    with open('config.py', 'w') as file:
        file.writelines(modified_lines)
    
    print('Credentials updated successfully.')

def update_Telegram(tg_token,tg_channel):
    with open('config.py', 'r') as file:
      lines = file.readlines()
  
    modified_lines = []
    for line in lines:
        if line.startswith('tg_token = '):
            modified_lines.append(f"tg_token = '{tg_token}'\n")
        elif line.startswith('tg_channel = '):
            modified_lines.append(f"tg_channel = '{tg_channel}'\n")
        else:
            modified_lines.append(line)
    
    with open('config.py', 'w') as file:
        file.writelines(modified_lines)
    
    print('Telegram credentials updated successfully.')

def write_output(output):
    # Write output to the file
    with open('output.txt', 'a', encoding='utf-8') as output_file:
        for line in output:
            output_file.write(line + '\n')